@@ -265,6 +265,12 @@ ags_sf2_synth_util_copy_s8(gint8 *buffer,
   guint k;
   guint l;
   gboolean pong_copy;
+
+  source_frame_count = 0;
+
+  source_samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
+  source_buffer_size = AGS_SOUNDCARD_DEFAULT_BUFFER_SIZE;
+  source_format = AGS_SOUNDCARD_DEFAULT_FORMAT;  
   
   ags_sound_resource_info(AGS_SOUND_RESOURCE(ipatch_sample),
 			  &source_frame_count,
@@ -553,6 +559,12 @@ ags_sf2_synth_util_copy_s16(gint16 *buffer,
   guint k;
   guint l;
   gboolean pong_copy;
+
+  source_frame_count = 0;
+
+  source_samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
+  source_buffer_size = AGS_SOUNDCARD_DEFAULT_BUFFER_SIZE;
+  source_format = AGS_SOUNDCARD_DEFAULT_FORMAT;  
   
   ags_sound_resource_info(AGS_SOUND_RESOURCE(ipatch_sample),
 			  &source_frame_count,
@@ -842,6 +854,12 @@ ags_sf2_synth_util_copy_s24(gint32 *buffer,
   guint l;
   gboolean pong_copy;
   
+  source_frame_count = 0;
+
+  source_samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
+  source_buffer_size = AGS_SOUNDCARD_DEFAULT_BUFFER_SIZE;
+  source_format = AGS_SOUNDCARD_DEFAULT_FORMAT;  
+
   ags_sound_resource_info(AGS_SOUND_RESOURCE(ipatch_sample),
 			  &source_frame_count,
 			  NULL, NULL);
@@ -1130,6 +1148,12 @@ ags_sf2_synth_util_copy_s32(gint32 *buffer,
   guint l;
   gboolean pong_copy;
   
+  source_frame_count = 0;
+
+  source_samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
+  source_buffer_size = AGS_SOUNDCARD_DEFAULT_BUFFER_SIZE;
+  source_format = AGS_SOUNDCARD_DEFAULT_FORMAT;  
+
   ags_sound_resource_info(AGS_SOUND_RESOURCE(ipatch_sample),
 			  &source_frame_count,
 			  NULL, NULL);
@@ -1418,6 +1442,12 @@ ags_sf2_synth_util_copy_s64(gint64 *buffer,
   guint l;
   gboolean pong_copy;
   
+  source_frame_count = 0;
+
+  source_samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
+  source_buffer_size = AGS_SOUNDCARD_DEFAULT_BUFFER_SIZE;
+  source_format = AGS_SOUNDCARD_DEFAULT_FORMAT;  
+
   ags_sound_resource_info(AGS_SOUND_RESOURCE(ipatch_sample),
 			  &source_frame_count,
 			  NULL, NULL);
@@ -1706,6 +1736,12 @@ ags_sf2_synth_util_copy_float(gfloat *buffer,
   guint l;
   gboolean pong_copy;
   
+  source_frame_count = 0;
+
+  source_samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
+  source_buffer_size = AGS_SOUNDCARD_DEFAULT_BUFFER_SIZE;
+  source_format = AGS_SOUNDCARD_DEFAULT_FORMAT;  
+
   ags_sound_resource_info(AGS_SOUND_RESOURCE(ipatch_sample),
 			  &source_frame_count,
 			  NULL, NULL);
@@ -1994,6 +2030,12 @@ ags_sf2_synth_util_copy_double(gdouble *buffer,
   guint l;
   gboolean pong_copy;
   
+  source_frame_count = 0;
+
+  source_samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
+  source_buffer_size = AGS_SOUNDCARD_DEFAULT_BUFFER_SIZE;
+  source_format = AGS_SOUNDCARD_DEFAULT_FORMAT;  
+
   ags_sound_resource_info(AGS_SOUND_RESOURCE(ipatch_sample),
 			  &source_frame_count,
 			  NULL, NULL);
@@ -2282,6 +2324,12 @@ ags_sf2_synth_util_copy_complex(AgsComplex *buffer,
   guint l;
   gboolean pong_copy;
   
+  source_frame_count = 0;
+
+  source_samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
+  source_buffer_size = AGS_SOUNDCARD_DEFAULT_BUFFER_SIZE;
+  source_format = AGS_SOUNDCARD_DEFAULT_FORMAT;  
+
   ags_sound_resource_info(AGS_SOUND_RESOURCE(ipatch_sample),
 			  &source_frame_count,
 			  NULL, NULL);
