@@ -81,7 +81,7 @@ struct AgsFunctionalPitchTestWave* ags_functional_pitch_test_alloc(GList *templa
   "\n"							\
   "[soundcard-0]\n"					\
   "backend=alsa\n"					\
-  "device=default\n"					\
+  "device=hw:CARD=PCH,DEV=0\n"				\
   "samplerate=44100\n"					\
   "buffer-size=1024\n"					\
   "pcm-channels=2\n"					\
@@ -367,9 +367,6 @@ ags_functional_pitch_test_pitch_up()
 
   gint i, j, k;
   gboolean success;
-
-  CU_ASSERT(1);
-  return;
   
   start_wave = NULL;
   
